# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-07 14:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_data_collections', '0004_webformpluginmodel'),
    ]

    operations = [
        migrations.AlterField(
            model_name='webformpluginmodel',
            name='title',
            field=models.CharField(blank=True, default='', help_text='Optional. If not set the web form title will be used.', max_length=255, verbose_name='Title'),
        ),
    ]
