# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-07 12:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('pcart_data_collections', '0003_datacollection_reversed_ordering'),
    ]

    operations = [
        migrations.CreateModel(
            name='WebFormPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='pcart_data_collections_webformpluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('show_title', models.BooleanField(default=True, verbose_name='Show title')),
                ('title', models.CharField(blank=True, default='', help_text='Optional. If not set the collection title will be used.', max_length=255, verbose_name='Title')),
                ('template_name', models.CharField(blank=True, default='default', max_length=100, verbose_name='Template name')),
                ('web_form', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='pcart_data_collections.WebForm', verbose_name='Web form')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]
