# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-06 08:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_data_collections', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='datacollection',
            name='active_field_label',
            field=models.CharField(blank=True, max_length=255, verbose_name='Active field label'),
        ),
        migrations.AddField(
            model_name='datacollection',
            name='title_field_label',
            field=models.CharField(blank=True, max_length=255, verbose_name='Title field label'),
        ),
    ]
