from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class PCartDataCollectionsConfig(AppConfig):
    name = 'pcart_data_collections'
    verbose_name = _('Data collections')
