# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-06 12:36
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_data_collections', '0002_auto_20171006_1125'),
    ]

    operations = [
        migrations.AddField(
            model_name='datacollection',
            name='reversed_ordering',
            field=models.BooleanField(default=False, verbose_name='Reversed ordering'),
        ),
    ]
